import * as React from 'react';
export declare type Route = {
    name: string;
    path: string;
    page: React.FunctionComponent;
};
export declare const RouterContext: React.Context<RouterContextType>;
declare type UseRouterType<TRouteParams> = {
    params: RouteParams<TRouteParams>;
    route: string;
    routeParams: RouteParams<TRouteParams>;
    routeName: string;
    pushRoute: (routeName: string, routeParams?: RouteParams) => void;
    urlByRoute: (routeName: string, routeParams?: RouteParams) => string;
};
export declare function useRouter<TRouteParams = {}>(beforeUnload?: BeforeUnload): UseRouterType<TRouteParams>;
declare function Router(props: RouterProviderType): any;
export declare const Link: React.FC<LinkProps>;
export declare type RouteParams<TRouteParams = {}> = {
    [key: string]: any;
} & TRouteParams;
export declare type StoreRoute = {
    name: string;
    path: string;
    pattern: string;
    keys?: {
        [key: string]: number;
    };
    page: React.FunctionComponent;
};
export declare type StoreRoutes = {
    [key: string]: StoreRoute;
};
export declare type LinkProps = Omit<React.AnchorHTMLAttributes<HTMLAnchorElement>, 'href'> & {
    route: string;
    params?: RouteParams;
};
declare type BeforeUnload = (run: (value: unknown) => void) => void;
export declare type RouterContextType = {
    path: string;
    routes: StoreRoutes;
    beforeUnloads: {
        [id: number]: BeforeUnload;
    };
    setPath: (value: string) => void;
    addBeforeUnload: (id: number, value: BeforeUnload) => void;
    clearBeforeUpload: () => void;
};
export declare type RouterProviderType = {
    routes: Route[];
    notFoundPage?: React.ReactNode;
    onLoadPage?: (routeName: string) => void;
};
export declare function connect(routes: Route[]): StoreRoutes;
export declare function getUrlByRoute(routes: StoreRoutes, routeName: string, routeParams?: RouteParams): string;
export declare function getRouteByUrl(routes: StoreRoutes, url: string): [StoreRoute, RouteParams] | [];
export default Router;
